<?php
namespace JHodges\GiteaBotPHPUnit;

use \JHodges\GiteaBot\Client;

/* creates a new issue for each failed test
* if it already exists then report is added as a comment
* if it passes then the issue is closed
*/

final class PostFailedTests{

    private $repo=null;

    function __construct($xml_path){
        $this->xml = simplexml_load_file($xml_path);
    }

    public function post($url, $user, $pass, $repoUser, $repo){
        if(!$url) return;

        // open connection and repo
        $client=new Client($url, $user, $pass, true);
        $this->repo=$client->getRepo($repoUser, $repo);

        $this->process($this->xml);
    }


    private function fail($name,$att,$error,$message){
        //stick the error type into the atts array
        $att['result']=$error;
        //trim the message
        $message=trim($message);

        // get any existing issue
        $issue=$this->repo->getIssues(['q' => $name])[0]??null;

        // if existing issue, is the error the same?
        if($issue){
            if( $this->doesMessageMatch($issue->body,$message,$att) ){
                $issue->addComment("Failed again. Same errors [above]($issue->html_url#issue-{$issue->id})");
                return;
            }else{
                foreach($issue->getComments() as $comment){
                    if( $this->doesMessageMatch($comment->body,$message,$att) ){
                        $issue->addComment("Failed again. Same errors as [above]($issue->html_url#issuecomment-{$comment->id})");
                        return;
                    }
                }
            }
        }

        //so its a new iss or changed error, lets upload screenshot and compose the body
        $issueBody='';
        if(file_exists("/tmp/$name.png")){
            $data=$this->repo->addReleaseAttachment(1,"/tmp/$name.png");
            $url=$data->browser_download_url;
            $issueBody.="\n![]($url)\n";
            unlink("/tmp/$name.png");
        }
        if($message) $issueBody.="```plain\n$message\n```\n";
        foreach($att as $k=>$v){
            $issueBody.=" * $k = $v\n";
        }

        //post the body
        if($issue){
            $issue->addComment($issueBody);
        }else{
            $issue=$this->repo->createIssue([
                'title'=>'Failed test '.$name,
                'body'=>$issueBody
            ]);
        }

    }

    private function doesMessageMatch($body,$message,$att){
        // extract the old message
        preg_match('#```plain\n(.*)\n```#s',$body,$match);
        $messageOld=$match[1]??'';
        // return false if not the same
        if( $message != $messageOld ){
            return false;
        }

        // extract the old atts
        $attOld=[];
        preg_match_all('# \* (.*?) = (.*)#',$body,$m);
        foreach($m[0]??[] as $k=>$v){
            $attOld[$m[1][$k]]=$m[2][$k];
        }
        // make sure all current attributes (except time) are the same as old
        // return false on first mis-match
        foreach($att as $k=>$v){
            if($k=='time') continue;
            if(($attOld[$k]??null) != $v){
                return false;
            }
        }

        // all matches, error message is the same
        return true;
    }

    private function success($name,$att){
        $issue=$this->repo->getIssues(['q' => $name])[0]??null;

        if($issue){
            $issue->state='closed';
            $issue->save();
        }
    }

    private function process($item){
        if(isset($item->testcase)){
            foreach($item->testcase as $testcase) {
                $att=$testcase->attributes();
                $name=$testcase->attributes()->class[0].'::'.$testcase->attributes()->name[0];
                $fail=false;
                foreach($testcase as $k=>$v){
                    $this->fail($name,$att,$k,$v);
                    $fail=true;
                }
                if(!$fail){
                    $this->success($name,$att);
                }
            }
        }
        if(isset($item->testsuite)){
            foreach($item->testsuite as $testsuite) {
                $this->process($testsuite);
            }
        }
    }

}
