<?php
namespace JHodges\GiteaBotPHPUnit;

use \JHodges\GiteaBot\Client;

final class PostFullReport{

    private $report;
    private $level=0;
    private $failCount=0;

    function __construct($xml_path){
        $this->xml = simplexml_load_file($xml_path);
    }

    public function post($url, $user, $pass, $repoUser, $repo){
        if(!$url) return;
        // open connection and repo
        $this->client=new Client($url, $user, $pass);
        $this->repo=$this->client->getRepo($repoUser, $repo);

        $this->process($this->xml); //process xml and upload screenshots
        // create the issue
        if(trim($this->report)){
            $issue=$this->repo->createIssue([
                'title'=>'Test Results',
                'body'=>$this->report
            ]);
        }

    }

    public function getFailCount(){
        return $this->failCount;
    }

    private function process($item){
        $this->level++;
        if(isset($item->testcase)){
            foreach($item->testcase as $testcase) {
                $att=$testcase->attributes();
                $failmsg='';
                foreach($testcase as $k=>$v){
                    $failmsg="(**$k**)\n";
                    if(trim($v)) $failmsg.="```plain\n$v\n```\n";
                }
                if($failmsg){
                    $this->failCount++;
                    $this->report.=" * [ ] {$att->name[0]} $failmsg\n";
                }else{
                    $this->report.=" * [x] {$att->name[0]}\n";
                }
                $name=$testcase->attributes()->class[0].'::'.$testcase->attributes()->name[0];
                if(file_exists("/tmp/$name.png")){
                    $data=$this->repo->addReleaseAttachment(1,"/tmp/$name.png");
                    $url=$data->browser_download_url;
                    $img="\n![]($url)\n";
                    unlink("/tmp/$name.png");
                    $this->report.="$img\n";
                }
            }
        }
        if(isset($item->testsuite)){
            foreach($item->testsuite as $testsuite) {
                $att=$testsuite->attributes();
                $this->report.=str_repeat('#',$this->level)." {$att->name[0]}\n".
                "tests: {$att->tests[0]}".
                ", assertions: {$att->assertions[0]}".
                ", errors: {$att->errors[0]}".
                ", failures: {$att->failures[0]}".
                ", skipped: {$att->skipped[0]}".
                ", time: {$att->time[0]}".
                "\n";
                $this->process($testsuite);
            }
        }
        $this->level--;
    }

}
